# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
import grpc

import entity_pb2 as entity__pb2


class ParserStub(object):
  # missing associated documentation comment in .proto file
  pass

  def __init__(self, channel):
    """Constructor.

    Args:
      channel: A grpc.Channel.
    """
    self.parse = channel.unary_unary(
        '/nudnik.entity.Parser/parse',
        request_serializer=entity__pb2.Request.SerializeToString,
        response_deserializer=entity__pb2.Response.FromString,
        )


class ParserServicer(object):
  # missing associated documentation comment in .proto file
  pass

  def parse(self, request, context):
    # missing associated documentation comment in .proto file
    pass
    context.set_code(grpc.StatusCode.UNIMPLEMENTED)
    context.set_details('Method not implemented!')
    raise NotImplementedError('Method not implemented!')


def add_ParserServicer_to_server(servicer, server):
  rpc_method_handlers = {
      'parse': grpc.unary_unary_rpc_method_handler(
          servicer.parse,
          request_deserializer=entity__pb2.Request.FromString,
          response_serializer=entity__pb2.Response.SerializeToString,
      ),
  }
  generic_handler = grpc.method_handlers_generic_handler(
      'nudnik.entity.Parser', rpc_method_handlers)
  server.add_generic_rpc_handlers((generic_handler,))
