# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: entity.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='entity.proto',
  package='nudnik.entity',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0c\x65ntity.proto\x12\rnudnik.entity\"w\n\x04Load\x12/\n\tload_type\x18\x01 \x01(\x0e\x32\x1c.nudnik.entity.Load.LoadType\x12\r\n\x05value\x18\x02 \x01(\t\"/\n\x08LoadType\x12\x07\n\x03rtt\x10\x00\x12\x08\n\x04rttr\x10\x01\x12\x07\n\x03\x63pu\x10\x02\x12\x07\n\x03mem\x10\x03\"\xb2\x01\n\x07Request\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x11\n\tstream_id\x18\x02 \x01(\x05\x12\x13\n\x0bsequence_id\x18\x03 \x01(\x05\x12\x12\n\nmessage_id\x18\x04 \x01(\x05\x12\r\n\x05\x63time\x18\x05 \x01(\x03\x12\r\n\x05rtime\x18\x06 \x01(\x03\x12\x0e\n\x06rcount\x18\x07 \x01(\x05\x12\x0c\n\x04meta\x18\x08 \x01(\x0c\x12!\n\x04load\x18\t \x03(\x0b\x32\x13.nudnik.entity.Load\"\x8c\x01\n\x08Response\x12\x37\n\x0bstatus_code\x18\x01 \x01(\x0e\x32\".nudnik.entity.Response.StatusCode\x12\r\n\x05ptime\x18\x02 \x01(\x03\"8\n\nStatusCode\x12\x06\n\x02OK\x10\x00\x12\x10\n\x0cSERVER_ERROR\x10\x01\x12\x10\n\x0c\x43LIENT_ERROR\x10\x02\x32\x44\n\x06Parser\x12:\n\x05parse\x12\x16.nudnik.entity.Request\x1a\x17.nudnik.entity.Response\"\x00\x62\x06proto3')
)



_LOAD_LOADTYPE = _descriptor.EnumDescriptor(
  name='LoadType',
  full_name='nudnik.entity.Load.LoadType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='rtt', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='rttr', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='cpu', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='mem', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=103,
  serialized_end=150,
)
_sym_db.RegisterEnumDescriptor(_LOAD_LOADTYPE)

_RESPONSE_STATUSCODE = _descriptor.EnumDescriptor(
  name='StatusCode',
  full_name='nudnik.entity.Response.StatusCode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OK', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SERVER_ERROR', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='CLIENT_ERROR', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=418,
  serialized_end=474,
)
_sym_db.RegisterEnumDescriptor(_RESPONSE_STATUSCODE)


_LOAD = _descriptor.Descriptor(
  name='Load',
  full_name='nudnik.entity.Load',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='load_type', full_name='nudnik.entity.Load.load_type', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='nudnik.entity.Load.value', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _LOAD_LOADTYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=31,
  serialized_end=150,
)


_REQUEST = _descriptor.Descriptor(
  name='Request',
  full_name='nudnik.entity.Request',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='nudnik.entity.Request.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stream_id', full_name='nudnik.entity.Request.stream_id', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='sequence_id', full_name='nudnik.entity.Request.sequence_id', index=2,
      number=3, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_id', full_name='nudnik.entity.Request.message_id', index=3,
      number=4, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ctime', full_name='nudnik.entity.Request.ctime', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rtime', full_name='nudnik.entity.Request.rtime', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='rcount', full_name='nudnik.entity.Request.rcount', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='meta', full_name='nudnik.entity.Request.meta', index=7,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='load', full_name='nudnik.entity.Request.load', index=8,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=153,
  serialized_end=331,
)


_RESPONSE = _descriptor.Descriptor(
  name='Response',
  full_name='nudnik.entity.Response',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status_code', full_name='nudnik.entity.Response.status_code', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ptime', full_name='nudnik.entity.Response.ptime', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _RESPONSE_STATUSCODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=334,
  serialized_end=474,
)

_LOAD.fields_by_name['load_type'].enum_type = _LOAD_LOADTYPE
_LOAD_LOADTYPE.containing_type = _LOAD
_REQUEST.fields_by_name['load'].message_type = _LOAD
_RESPONSE.fields_by_name['status_code'].enum_type = _RESPONSE_STATUSCODE
_RESPONSE_STATUSCODE.containing_type = _RESPONSE
DESCRIPTOR.message_types_by_name['Load'] = _LOAD
DESCRIPTOR.message_types_by_name['Request'] = _REQUEST
DESCRIPTOR.message_types_by_name['Response'] = _RESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Load = _reflection.GeneratedProtocolMessageType('Load', (_message.Message,), dict(
  DESCRIPTOR = _LOAD,
  __module__ = 'entity_pb2'
  # @@protoc_insertion_point(class_scope:nudnik.entity.Load)
  ))
_sym_db.RegisterMessage(Load)

Request = _reflection.GeneratedProtocolMessageType('Request', (_message.Message,), dict(
  DESCRIPTOR = _REQUEST,
  __module__ = 'entity_pb2'
  # @@protoc_insertion_point(class_scope:nudnik.entity.Request)
  ))
_sym_db.RegisterMessage(Request)

Response = _reflection.GeneratedProtocolMessageType('Response', (_message.Message,), dict(
  DESCRIPTOR = _RESPONSE,
  __module__ = 'entity_pb2'
  # @@protoc_insertion_point(class_scope:nudnik.entity.Response)
  ))
_sym_db.RegisterMessage(Response)



_PARSER = _descriptor.ServiceDescriptor(
  name='Parser',
  full_name='nudnik.entity.Parser',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=476,
  serialized_end=544,
  methods=[
  _descriptor.MethodDescriptor(
    name='parse',
    full_name='nudnik.entity.Parser.parse',
    index=0,
    containing_service=None,
    input_type=_REQUEST,
    output_type=_RESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_PARSER)

DESCRIPTOR.services_by_name['Parser'] = _PARSER

# @@protoc_insertion_point(module_scope)
